@@ -19,12 +19,12 @@ $p->push($p->factory(priority => 12, data => 'twelve-2'));
 $p->push($p->factory(priority => 20, data => 'twenty'), $p->factory(priority => 0, data => 'zero'));
 print "\$p->size()=", $p->size(), "\n";
 print "\$p->top():"; MyPrint->new()->function_operator($p->top());
-::foreach($p->begin(), $p->end(), MyPrint->new());
+for_each($p->begin(), $p->end(), MyPrint->new());
 print '$p->top()->priority(7);', "\n";
 print '$p->refresh();', "\n";
 $p->top()->priority(7);
 $p->refresh();
-::foreach($p->begin(), $p->end(), MyPrint->new());
+for_each($p->begin(), $p->end(), MyPrint->new());
 print "\$p->top():"; MyPrint->new()->function_operator($p->top());
 print '$p->pop();'. "\n";
 $p->pop();
